part of katana_routing;

/// Defining values that can maintain state in Scoped.
@immutable
abstract class ScopedValue<TResult> {
  /// Defining values that can maintain state in Scoped.
  const ScopedValue();

  /// Create a new state.
  ScopedValueState<TResult, ScopedValue<TResult>> createState();
}

/// State created from Container.
abstract class ScopedValueState<TResult,
    TScopedValue extends ScopedValue<TResult>> {
  WidgetRef? _ref;
  TScopedValue? _value;

  /// Current container.
  TScopedValue get value => _value!;

  /// When executed, [Scoped], [ScopedWidget] and [PageScopedWidget] will be rebuilt.
  ///
  /// [callback] is called before the rebuild is done.
  @protected
  @mustCallSuper
  void setState(VoidCallback callback) {
    callback.call();
    _ref?.rebuild();
  }

  /// Process when the scoped value is updated.
  @mustCallSuper
  void didUpdateValue(covariant TScopedValue oldValue) {}

  /// Called when the scoped value is created.
  @mustCallSuper
  void initValue() {}

  /// Called when the page is disposed.
  @mustCallSuper
  void dispose() {}

  /// Build and output the value.
  TResult build();
}
